unit Common;

{**********************************************************************}
{                                                                      }
{   Common                                                             }
{                                                                      }
{   data types, error handling and internal conversion                 }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, SysUtils, Forms;

{----------------------------------------------------------------------}
{   dialog version                                                     }
{----------------------------------------------------------------------}

const
  cnVersionHi           = 0;
  cnVersionLo           = 96;

{----------------------------------------------------------------------}
{   return codes                                                       }
{----------------------------------------------------------------------}

const
  idUser                = 65536;
  idHelpEx              = idUser + 1;
  idList                = idUser + 2;
  idOpen                = idUser + 3;
  idLoad                = idUser + 4;
  idSave                = idUser + 5;
  idRemove              = idUser + 6;
  idRemoveAll           = idUser + 7;
  idNext                = idUser + 8;
  idPrev                = idUser + 9;

{----------------------------------------------------------------------}
{   return codes (todo-list)                                           }
{----------------------------------------------------------------------}

const
  idTodoGoto            = 1;
  idTodoUpdate          = 101;
  idTodoEdit            = 102;
  idTodoDone            = 103;
  idTodoTodo            = 104;
  idTodoRemove          = 105;
  idTodoSetup           = 121;

{----------------------------------------------------------------------}
{   error codes                                                        }
{----------------------------------------------------------------------}

const
  slOK                  = 0;
  slInvalidID           = -1;
  slInvalidIndex        = -2;
  slInvalidLength       = -3;
  slUnhandledException  = -99;

{----------------------------------------------------------------------}
{   data types                                                         }
{----------------------------------------------------------------------}

const
  MaxSalStrLen = 255;                   // Maximum size of SAL strings
type
  PSalStr = ^TSalStr;                   // SAL string structure
  TSalStr = record
    Size: SmallInt;
    Data: array [0..MaxSalStrLen] of Char;
  end;

{----------------------------------------------------------------------}
{   utility routines                                                   }
{----------------------------------------------------------------------}

function CompareNumeric(N1, N2: Integer): Integer;

function GetSalStr(Str: PSalStr): string;
procedure SetSalStr(Str: PSalStr; Len: Integer; S: string);

function Tokenize(T: Char; var S: string): string;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

{----------------------------------------------------------------------}
{   compare two numbers                                                }
{----------------------------------------------------------------------}

function CompareNumeric(N1, N2: Integer): Integer;
begin
  if N1 < N2 then
    Result := -1
  else if N1 > N2 then
    Result := +1
  else
    Result := 0;
end;

{----------------------------------------------------------------------}
{   SAL to pascal strings                                              }
{----------------------------------------------------------------------}

function GetSalStr(Str: PSalStr): string;
begin
  SetString(Result, Str^.Data, Str^.Size);
end;

procedure SetSalStr(Str: PSalStr; Len: Integer; S: string);
begin
  if Len < Length(S) then S := Copy(S, 1, Len);
  Str^.Size := Length(S);
  Move(PChar(S)^, Str^.Data, Length(S));
end;

{----------------------------------------------------------------------}
{   a simple tokenizer                                                 }
{----------------------------------------------------------------------}

function Tokenize(T: Char; var S: string): string;
var
  P: Integer;
begin
  P := Pos(T,S);
  if P > 0 then
  begin
    Result := Copy(S,1,P-1);
    Delete(S,1,P);
  end else
  begin
    Result := S;
    S := '';
  end;
end;

end.

