unit DlgOpen;

{**********************************************************************}
{                                                                      }
{   DlgOpen                                                            }
{                                                                      }
{   common open dialogs                                                }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003 by DiK                                        }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Dialogs, Common;

function WinDlgOpenEx(
  FilesID: Integer; FileTypesId: Integer; Caption: PSalStr;
  var AsBinary: Integer; var ChangeDir: Integer): Integer; stdcall;

function WinDlgOpen(
  FilesID: Integer; FileTypesId: Integer; Caption: PSalStr): Integer; stdcall;

function WinDlgSaveAs(
  FilesID: Integer; FileTypesId: Integer; Caption: PSalStr): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  StrList, DlgOpenEx, FormMsgBox;

resourcestring
  rsError       = 'WinDlg Error';
  rsErrorText   = 'Open/Save dialog: invalid list ID';

{----------------------------------------------------------------------}

function ValidateStrings(ID: Integer): TStrings;
begin
  if not GetStringList(ID, Result) then
  begin
    DlgMsgBox(rsError, rsErrorText);
    Abort;
  end;
end;

{----------------------------------------------------------------------}

function DoDlgOpen(OpenDlg: TOpenDialog;
  FilesID: Integer; FileTypesID: Integer; Caption: PSalStr): Integer;
var
  I: Integer;
  InitName: string;
  FileTypes: string;
  FilesList: TStrings;
  FileTypesList: TStrings;
begin
  Result := idCancel;
  try
    FilesList := ValidateStrings(FilesID);
    FileTypesList := ValidateStrings(FileTypesID);

    if FilesList.Count > 0 then
    begin
      InitName := FilesList[0];
      if not FileExists(InitName) then InitName := '';
    end else
      InitName := '';

    if FileTypesList.Count > 0 then
    begin
      FileTypes := FileTypesList[0];
      for I := 1 to FileTypesList.Count-1 do
        FileTypes := FileTypes + '|' + FileTypesList[I];
    end else
      FileTypes := 'Any File (*.*)|(*.*)';

    with OpenDlg do
    begin
      Options := Options + [ofNoChangeDir, ofPathMustExist];
      Title := GetSalStr(Caption);
      InitialDir := GetCurrentDir;
      FileName := InitName;
      Filter := FileTypes;
      if Execute then
      begin
        FilesList.Assign(Files);
        Result := idOK
      end;
    end;
  except
  end;
end;

{----------------------------------------------------------------------}

function WinDlgOpenEx(
  FilesID: Integer; FileTypesId: Integer;
  Caption: PSalStr; var AsBinary: Integer; var ChangeDir: Integer): Integer;
var
  OpenDlg: TTseOpenDialog;
begin
  OpenDlg := TTseOpenDialog.Create(nil);
  try
    OpenDlg.Options := OpenDlg.Options + [ofAllowMultiSelect];
    Result := DoDlgOpen(OpenDlg, FilesID, FileTypesID, Caption);
    AsBinary := Integer(OpenDlg.AsBinary);
  finally
    OpenDlg.Free;
  end;
end;

{----------------------------------------------------------------------}

function WinDlgOpen(
  FilesID: Integer; FileTypesId: Integer; Caption: PSalStr): Integer;
var
  OpenDlg: TOpenDialog;
begin
  OpenDlg := TOpenDialog.Create(nil);
  try
    OpenDlg.Options := OpenDlg.Options + [ofFileMustExist];
    Result := DoDlgOpen(OpenDlg, FilesID, FileTypesID, Caption);
  finally
    OpenDlg.Free;
  end;
end;

{----------------------------------------------------------------------}

function WinDlgSaveAs(
  FilesID: Integer; FileTypesId: Integer; Caption: PSalStr): Integer;
var
  SaveDlg: TSaveDialog;
begin
  SaveDlg := TSaveDialog.Create(nil);
  try
    SaveDlg.Options := SaveDlg.Options + [ofOverwritePrompt, ofFileMustExist];
    Result := DoDlgOpen(SaveDlg, FilesID, FileTypesID, Caption);
  finally
    SaveDlg.Free;
  end;
end;

{----------------------------------------------------------------------}


end.
