unit FormBookmarks;

{**********************************************************************}
{                                                                      }
{   FormOpts                                                           }
{                                                                      }
{   options dialog                                                     }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,
  Common, FormDlgs, Menus, StdCtrls, ComCtrls;

type
  TFrmBookmarks = class(TFrmDlgs)
    ListView: TListView;
    BtnDel: TButton;
    BtnDelAll: TButton;
    procedure ListViewKeyDown(
      Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListViewChange(
      Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure ListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListViewDblClick(Sender: TObject);
    procedure ListViewCompare(Sender: TObject;
      Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure BtnDelClick(Sender: TObject);
    procedure BtnDelAllClick(Sender: TObject);
  private
    SortColumn: Integer;
    DelIDs: string;
  public
    procedure SetBookmarks(ID: Integer);
  end;

function WinDlgBookmarks(BookmarkList: Integer;
  Selected: PSalStr; MaxSel: Integer; var SortCol: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

{$R *.dfm}

{----------------------------------------------------------------------}

function WinDlgBookmarks(BookmarkList: Integer;
  Selected: PSalStr; MaxSel: Integer; var SortCol: Integer): Integer;
var
  Form: TFrmBookmarks;
begin
  Form := TFrmBookmarks.Create(nil);
  try
    try
      with Form do
      begin
        SetBookmarks(BookmarkList);
        ListView.CustomSort(nil, SortCol);
        Result := Execute;
        SortCol := SortColumn;
        case Result of
          idOk:     SetSalStr(Selected, MaxSel, ListView.Selected.Caption);
          idRemove: SetSalStr(Selected, MaxSel, DelIDs);
          else      SetSalStr(Selected, MaxSel, '');
        end;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmBookmarks.SetBookmarks(ID: Integer);
var
  I: Integer;
  S,T: string;
  BM: TStrings;
begin
  BM := ValidateStrings(ID);
  for I := 0 to BM.Count-1 do
    with ListView.Items.Add do
    begin
      S := BM[I];
      T := Tokenize(#9,S);
      Caption := T;
      if T[1] in ['q'..'z'] then T := Chr(Ord(T[1]) - 65) else T := '';
      SubItems.Add(T);
      SubItems.Add('');
      SubItems.Add(Tokenize(#9,S));
      SubItems.Add(Tokenize(#9,S));
      SubItems[1] := Tokenize(#9,S);
      SubItems.Add(Tokenize(#9,S));
    end;
end;

{----------------------------------------------------------------------}

procedure TFrmBookmarks.ListViewKeyDown(
  Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_DELETE) and (BtnDel.Enabled) then BtnDelClick(Sender);
end;

procedure TFrmBookmarks.ListViewChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
var
  Sel: Boolean;
begin
  Sel := ListView.Selected <> nil;
  BtnOk.Enabled := Sel;
  BtnDel.Enabled := Sel;
end;

procedure TFrmBookmarks.ListViewColumnClick(
  Sender: TObject; Column: TListColumn);
begin
  ListView.CustomSort(nil, Column.Index);
end;

procedure TFrmBookmarks.ListViewDblClick(Sender: TObject);
begin
  ModalResult := idOk;
end;

procedure TFrmBookmarks.ListViewCompare(Sender: TObject;
  Item1, Item2: TListItem; Data: Integer; var Compare: Integer);

  function ComparePosition: Integer;
  begin
    try
      Result := AnsiCompareText(Item1.SubItems[1], Item2.SubItems[1]);
      if Result = 0 then
        Result := CompareNumeric(
          StrToInt(Item1.SubItems[2]), StrToInt(Item2.SubItems[2]));
      if Result = 0 then
        Result := CompareNumeric(
          StrToInt(Item1.SubItems[3]), StrToInt(Item2.SubItems[3]));
    except
      Result := 0;
    end;
  end;

begin
  SortColumn := Data;
  case Data of
    0,1:    Compare := AnsiCompareText(Item1.Caption, Item2.Caption);
    2,3,4:  Compare := ComparePosition;
    else    Compare := AnsiCompareText(Item1.SubItems[4], Item2.SubItems[4]);
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmBookmarks.BtnDelClick(Sender: TObject);
var
  I: Integer;
begin
  for I := ListView.Items.Count - 1 downto 0 do
    if ListView.Items[I].Selected then
    begin
      DelIDs := DelIDs + ListView.Items[I].Caption;
      ListView.Items.Delete(I);
    end;
  ModalResult := idRemove;
end;

procedure TFrmBookmarks.BtnDelAllClick(Sender: TObject);
begin
  ModalResult := idRemoveAll;
end;

end.
