unit FormFind;

{**********************************************************************}
{                                                                      }
{   FormFind                                                           }
{                                                                      }
{   find dialog                                                        }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls,
  FormDlgs, Menus, ExtCtrls;

type
  TFrmFind = class(TFrmDlgs)
    LblFind: TLabel;
    CmbFind: TComboBox;
    GrpScope: TGroupBox;
    ChkAllFiles: TCheckBox;
    ChkGlobal: TCheckBox;
    ChkLocal: TCheckBox;
    ChkCurrLine: TCheckBox;
    BvlScope: TBevel;
    ChkBegLine: TCheckBox;
    ChkEndLine: TCheckBox;
    GrpDirect: TGroupBox;
    ChkBack: TCheckBox;
    GrpOptions: TGroupBox;
    ChkView: TCheckBox;
    ChkCase: TCheckBox;
    ChkWords: TCheckBox;
    ChkRegEx: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    fFindList: TStrings;
    fOptsList: TStrings;
  protected
    function GetOptions: string; virtual;
    procedure SetOptions(Opts: string); virtual;
  public
    procedure SetFindList(ID: Integer);
    procedure SetOptsList(ID: Integer);
  end;

function WinDlgFind(FindHist, FindOptsHist: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  StrList;

{$R *.dfm}

{----------------------------------------------------------------------}

function WinDlgFind(FindHist, FindOptsHist: Integer): Integer;
var
  Form: TFrmFind;
begin
  Form := TFrmFind.Create(nil);
  try
    try
      with Form do
      begin
        SetFindList(FindHist);
        SetOptsList(FindOptsHist);
        Result := Execute;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmFind.SetFindList(ID: Integer);
begin
  fFindList := ValidateStrings(ID);
end;

procedure TFrmFind.SetOptsList(ID: Integer);
begin
  fOptsList := ValidateStrings(ID);
end;

{----------------------------------------------------------------------}

function TFrmFind.GetOptions: string;
begin
  Result := '';
  if ChkAllFiles.Checked then Result := Result + 'a';
  if ChkGlobal.Checked then Result := Result + 'g';
  if ChkLocal.Checked then Result := Result + 'l';
  if ChkCurrLine.Checked then Result := Result + 'c';
  if ChkBegLine.Checked then Result := Result + '^';
  if ChkEndLine.Checked then Result := Result + '$';
  if ChkBack.Checked then Result := Result + 'b';
  if ChkView.Checked then Result := Result + 'v';
  if ChkCase.Checked then Result := Result + 'i';
  if ChkWords.Checked then Result := Result + 'w';
  if ChkRegEx.Checked then Result := Result + 'x';
end;

procedure TFrmFind.SetOptions(Opts: string);
begin
  ChkAllFiles.Checked := Pos('a', Opts) > 0;
  ChkGlobal.Checked := Pos('g', Opts) > 0;
  ChkLocal.Checked := Pos('l', Opts) > 0;
  ChkCurrLine.Checked := Pos('c', Opts) > 0;
  ChkBegLine.Checked := Pos('^', Opts) > 0;
  ChkEndLine.Checked := Pos('$', Opts) > 0;
  ChkBack.Checked := Pos('b', Opts) > 0;
  ChkView.Checked := Pos('v', Opts) > 0;
  ChkCase.Checked := Pos('i', Opts) > 0;
  ChkWords.Checked := Pos('w', Opts) > 0;
  ChkRegEx.Checked := Pos('x', Opts) > 0;
end;

{----------------------------------------------------------------------}

procedure TFrmFind.FormShow(Sender: TObject);
begin
  InitComboBox(CmbFind, fFindList);
  if fOptsList.Count > 0 then SetOptions(LowerCase(fOptsList[0]));
end;

procedure TFrmFind.FormHide(Sender: TObject);
begin
  fFindList.Insert(0, CmbFind.Text);
  fOptsList.Insert(0, GetOptions);
end;

end.
