unit FormFindAndDo;

{**********************************************************************}
{                                                                      }
{   FormFindAndDo                                                      }
{                                                                      }
{   find dialog                                                        }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{   Action codes are defined in find&do macro.                         }
{   If macro changes, tags of radio buttons must be adapted.           }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Graphics,
  StdCtrls, ExtCtrls, Common, FormDlgs, Menus, Buttons;

type
  TFrmFindAndDo = class(TFrmDlgs)
    LblFind: TLabel;
    CmbFind: TComboBox;
    LblMacro: TLabel;
    CmbMacro: TComboBox;
    BtnMacro: TBitBtn;
    GrpActions: TGroupBox;
    RadCount: TRadioButton;
    RadDel: TRadioButton;
    RadCut: TRadioButton;
    RadCopy: TRadioButton;
    RadMacro: TRadioButton;
    RadKey: TRadioButton;
    RadKeep: TRadioButton;
    RadQuit: TRadioButton;
    GrpScope: TGroupBox;
    ChkAllFiles: TCheckBox;
    ChkGlobal: TCheckBox;
    ChkLocal: TCheckBox;
    GrpOptions: TGroupBox;
    ChkPrompt: TCheckBox;
    ChkCase: TCheckBox;
    ChkWords: TCheckBox;
    ChkRegEx: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure BtnMacroClick(Sender: TObject);
    procedure RadCountClick(Sender: TObject);
  private
    fAction: Integer;
    fFindList: TStrings;
    fOptsList: TStrings;
    fMacsList: TStrings;
  protected
    function GetOptions: string; virtual;
    procedure SetOptions(Opts: string); virtual;
  end;

function WinDlgFindAndDo(
  FindHist, FindOptsHist, ExecMacsHist: Integer;
  var FindAction: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

{$R *.dfm}

{----------------------------------------------------------------------}

function WinDlgFindAndDo(
  FindHist, FindOptsHist, ExecMacsHist: Integer;
  var FindAction: Integer): Integer;
var
  Form: TFrmFindAndDo;
begin
  Form := TFrmFindAndDo.Create(nil);
  try
    try
      with Form do
      begin
        fAction := FindAction;
        fFindList := ValidateStrings(FindHist);
        fOptsList := ValidateStrings(FindOptsHist);
        fMacsList := ValidateStrings(ExecMacsHist);
        Result := Execute;
        FindAction := fAction;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

function TFrmFindAndDo.GetOptions: string;
begin
  Result := '';
  if ChkAllFiles.Checked then Result := Result + 'a';
  if ChkGlobal.Checked then Result := Result + 'g';
  if ChkLocal.Checked then Result := Result + 'l';
  if ChkPrompt.Checked then Result := Result + 'n';
  if ChkCase.Checked then Result := Result + 'i';
  if ChkWords.Checked then Result := Result + 'w';
  if ChkRegEx.Checked then Result := Result + 'x';
end;

procedure TFrmFindAndDo.SetOptions(Opts: string);
begin
  ChkAllFiles.Checked := Pos('a', Opts) > 0;
  ChkGlobal.Checked := Pos('g', Opts) > 0;
  ChkLocal.Checked := Pos('l', Opts) > 0;
  ChkPrompt.Checked := Pos('n', Opts) > 0;
  ChkCase.Checked := Pos('i', Opts) > 0;
  ChkWords.Checked := Pos('w', Opts) > 0;
  ChkRegEx.Checked := Pos('x', Opts) > 0;
end;

{----------------------------------------------------------------------}

procedure TFrmFindAndDo.FormShow(Sender: TObject);
var
  I: Integer;
begin
  InitComboBox(CmbFind, fFindList);
  InitComboBox(CmbMacro, fMacsList);
  if fOptsList.Count > 0 then SetOptions(LowerCase(fOptsList[0]));
  for I := 0 to ComponentCount-1 do
    if Components[I] is TRadioButton then
      if Components[I].Tag = fAction then
      begin
        TRadioButton(Components[I]).Checked := True;
        Break;
      end;
end;

procedure TFrmFindAndDo.FormHide(Sender: TObject);
begin
  fFindList.Insert(0, CmbFind.Text);
  fOptsList.Insert(0, GetOptions);
  fMacsList.Insert(0, CmbMacro.Text);
end;

{----------------------------------------------------------------------}

procedure TFrmFindAndDo.BtnMacroClick(Sender: TObject);
begin
  ModalResult := idOpen;
end;

procedure TFrmFindAndDo.RadCountClick(Sender: TObject);
var
  Enabled: Boolean;
begin
  fAction := (Sender as TRadioButton).Tag;
  Enabled := Sender = RadMacro;
  LblMacro.Enabled := Enabled;
  CmbMacro.Enabled := Enabled;
  BtnMacro.Enabled := Enabled;
  if Enabled then
    CmbMacro.Color := clWindow
  else
    CmbMacro.Color := clBtnFace;
end;

end.
