unit FormFolder;

{**********************************************************************}
{                                                                      }
{   FormFolder                                                         }
{                                                                      }
{   browse for folder dialog                                           }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{   Remark:                                                            }
{   This form produces an error when loaded into the form editor.      }
{   TStShellTreeView has troubles being loaded into an inherited       }
{   form. This error can be ignored since it doesn't destroy the       }
{   form and doesn't happen at runtime.                                }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ComCtrls,
  FormDlgs, StShlCtl, Common, Menus;

type
  TFrmFolder = class(TFrmDlgs)
    ShellTree: TStShellTreeView;
    BtnStartup: TButton;
    BtnCurrent: TButton;
    BtnRecent: TButton;
    procedure FormShow(Sender: TObject);
    procedure ShellTreeFolderSelected(Sender: TObject; Folder: TStShellFolder);
    procedure BtnStartupClick(Sender: TObject);
    procedure BtnCurrentClick(Sender: TObject);
    procedure BtnRecentClick(Sender: TObject);
  private
    fHistList: TStrings;
    fCurrFolder: string;
    fStartFolder: string;
    procedure SelectFolder(Folder: string);
  public
    procedure SetHistList(ID: Integer);
    procedure SetStartFolder(Folder: string);
  end;

function DlgPickFolder(var Folder: string; FolderHist: Integer): Integer;

function WinDlgPickFolder(
  Folder: PSalStr; MaxLen: Integer; FolderHist: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  FormStrList;

resourcestring
  rsRecentFolders = 'Recent Folders';

{$R *.dfm}

{----------------------------------------------------------------------}

function DlgPickFolder(var Folder: string; FolderHist: Integer): Integer;
var
  Form: TFrmFolder;
begin
  Form := TFrmFolder.Create(nil);
  try
    try
      with Form do
      begin
        if FolderHist <> slInvalidID then
        begin
          SetHistList(FolderHist);
          BtnRecent.Enabled := True;
        end;
        SetStartFolder(Folder);
        SelectFolder(Folder);
        Result := Execute;
        Folder := ShellTree.SelectedFolder.Path;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

function WinDlgPickFolder(
  Folder: PSalStr; MaxLen: Integer; FolderHist: Integer): Integer;
var
  S: string;
begin
  S := GetSalStr(Folder);
  Result := DlgPickFolder(S, FolderHist);
  SetSalStr(Folder, MaxLen, S);
end;

{----------------------------------------------------------------------}

procedure TFrmFolder.SelectFolder(Folder: string);
begin
  try
    ShellTree.SelectFolder(Folder);
  except
    ShellTree.SelectFolder('');
  end;
end;

procedure TFrmFolder.SetHistList(ID: Integer);
begin
  fHistList := ValidateStrings(ID);
end;

procedure TFrmFolder.SetStartFolder(Folder: string);
begin
  fStartFolder := Folder;
end;

procedure TFrmFolder.FormShow(Sender: TObject);
begin
  fCurrFolder := GetCurrentDir;
end;

procedure TFrmFolder.ShellTreeFolderSelected(
  Sender: TObject; Folder: TStShellFolder);
begin
  BtnOk.Enabled := Assigned(Folder) and Folder.IsFileFolder;
end;

procedure TFrmFolder.BtnStartupClick(Sender: TObject);
begin
  SelectFolder(fStartFolder);
end;

procedure TFrmFolder.BtnCurrentClick(Sender: TObject);
begin
  SelectFolder(fCurrFolder);
end;

procedure TFrmFolder.BtnRecentClick(Sender: TObject);
var
  I: Integer;
  Form: TFrmStrList;
begin
  Form := TFrmStrList.Create(Self);
  try
    with Form do
    begin
      Caption := rsRecentFolders;
      BtnHelp.Visible := False;
      LstBox.Items.AddStrings(fHistList);
      if Execute = idOk then
      begin
        I := LstBox.ItemIndex;
        if I >= 0 then SelectFolder(LstBox.Items[I]);
      end;
    end;
  finally
    Form.Free;
  end;
end;

end.
