unit FormGrep;

{**********************************************************************}
{                                                                      }
{   FormFindAndDo                                                      }
{                                                                      }
{   find dialog                                                        }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls,
  FormDlgs, Menus;

type
  TFrmGrep = class(TFrmDlgs)
    LblFind: TLabel;
    LblFile: TLabel;
    LblExcl: TLabel;
    LblPath: TLabel;
    CmbFind: TComboBox;
    CmbFile: TComboBox;
    CmbExcl: TComboBox;
    CmbPath: TComboBox;
    GrpScope: TRadioGroup;
    GrpViewOpts: TGroupBox;
    ChkNamesOnly: TCheckBox;
    GrpFindOps: TGroupBox;
    ChkRegEx: TCheckBox;
    ChkCase: TCheckBox;
    ChkWords: TCheckBox;
    GrpGrepOpts: TGroupBox;
    ChkSubDirs: TCheckBox;
    BtnAscii: TButton;
    BtnPath: TButton;
    BtnExHelp: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure GrpScopeClick(Sender: TObject);
    procedure BtnAsciiClick(Sender: TObject);
    procedure BtnPathClick(Sender: TObject);
    procedure BtnExHelpClick(Sender: TObject);
  private
    fFindList: TStrings;
    fFileList: TStrings;
    fExclList: TStrings;
    fPathList: TStrings;
    fOptsList: TStrings;
  protected
    function GetOptions: string; virtual;
    procedure SetOptions(Opts: string); virtual;
  public
    procedure SetFindList(ID: Integer);
    procedure SetFileList(ID: Integer);
    procedure SetExclList(ID: Integer);
    procedure SetPathList(ID: Integer);
    procedure SetOptsList(ID: Integer);
  end;

function WinDlgGrep(FindHist, FileHist, ExclHist, PathHist, OptsHist: Integer;
  GrepDisk: Boolean): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  ShellApi, Common, FormFolder, FormMsgBox;

resourcestring
  rsChartError  = 'Character-Map not found';
  rsNoHelp      = 'Sorry!'#13'Cannot display help right now!';
//  rsPickMessage = 'Pick root folder to search for files';
{$R *.dfm}

{----------------------------------------------------------------------}

function WinDlgGrep(
  FindHist, FileHist, ExclHist, PathHist, OptsHist: Integer;
  GrepDisk: Boolean): Integer;
var
  Form: TFrmGrep;
begin
  Form := TFrmGrep.Create(nil);
  try
    try
      with Form do
      begin
        SetFindList(FindHist);
        SetFileList(FileHist);
        SetExclList(ExclHist);
        SetPathList(PathHist);
        SetOptsList(OptsHist);
        if not GrepDisk then GrpScope.ItemIndex := 2;
        Result := Execute;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmGrep.SetFindList(ID: Integer);
begin
  fFindList := ValidateStrings(ID);
end;

procedure TFrmGrep.SetFileList(ID: Integer);
begin
  fFileList := ValidateStrings(ID);
end;

procedure TFrmGrep.SetExclList(ID: Integer);
begin
  fExclList := ValidateStrings(ID);
end;

procedure TFrmGrep.SetPathList(ID: Integer);
begin
  fPathList := ValidateStrings(ID);
end;

procedure TFrmGrep.SetOptsList(ID: Integer);
begin
  fOptsList := ValidateStrings(ID);
end;

{----------------------------------------------------------------------}

function TFrmGrep.GetOptions: string;
begin
  Result := '';
  if ChkRegEx.Checked then Result := Result + 'x';
  if ChkCase.Checked then Result := Result + 'i';
  if ChkWords.Checked then Result := Result + 'w';
  if ChkNamesOnly.Checked then Result := Result + 'n';
  if ChkSubDirs.Checked then Result := Result + 's';
  case GrpScope.ItemIndex of
    0: Result := Result + 'd';
    1: Result := Result + 'm';
    2: Result := Result + 'c';
  end;
end;

procedure TFrmGrep.SetOptions(Opts: string);
begin
  ChkRegEx.Checked := Pos('x', Opts) > 0;
  ChkCase.Checked := Pos('i', Opts) > 0;
  ChkWords.Checked := Pos('w', Opts) > 0;
  ChkNamesOnly.Checked := Pos('n', Opts) > 0;
  ChkSubDirs.Checked := Pos('s', Opts) > 0;
  if GrpScope.ItemIndex > 0 then
  begin
    if Pos('d', Opts) > 0 then
      GrpScope.ItemIndex := 0
    else if Pos('m', Opts) > 0 then
      GrpScope.ItemIndex := 1;
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmGrep.FormShow(Sender: TObject);
begin
  InitComboBox(CmbFind, fFindList);
  InitComboBox(CmbFile, fFileList);
  InitComboBox(CmbExcl, fExclList);
  InitComboBox(CmbPath, fPathList);
  if fOptsList.Count > 0 then SetOptions(LowerCase(fOptsList[0]));
end;

procedure TFrmGrep.FormHide(Sender: TObject);
begin
  fFindList.Insert(0, CmbFind.Text);
  fFileList.Insert(0, CmbFile.Text);
  fExclList.Insert(0, CmbExcl.Text);
  fPathList.Insert(0, CmbPath.Text);
  fOptsList.Insert(0, GetOptions);
end;

{----------------------------------------------------------------------}

procedure TFrmGrep.GrpScopeClick(Sender: TObject);
var
  Enabled: Boolean;
begin
  Enabled := GrpScope.ItemIndex = 0;
  LblFile.Enabled := Enabled;
  LblExcl.Enabled := Enabled;
  LblPath.Enabled := Enabled;
  CmbFile.Enabled := Enabled;
  CmbExcl.Enabled := Enabled;
  CmbPath.Enabled := Enabled;
  ChkNamesOnly.Enabled := Enabled;
  ChkSubDirs.Enabled := Enabled;
  BtnPath.Enabled := Enabled;
end;

{----------------------------------------------------------------------}

procedure TFrmGrep.BtnAsciiClick(Sender: TObject);
var
  H: THandle;
begin
  H := ShellExecute(0, 'open', 'charmap', nil, nil, SW_SHOW);
  if H <= 32 then DlgMsgBox(Caption, rsChartError);
end;

procedure TFrmGrep.BtnPathClick(Sender: TObject);
var
  RC: Integer;
  Folder: string;
begin
  Folder := CmbPath.Text;
  RC := DlgPickFolder(Folder, slInvalidID);
  if RC = idOK then
  begin
    CmbPath.Text := Folder;
    CmbFind.SetFocus;
  end else
  if RC = idHelp then
    DlgMsgBox(Caption, rsNoHelp);
end;

procedure TFrmGrep.BtnExHelpClick(Sender: TObject);
begin
  ModalResult := idHelpEx;
end;

end.
