unit FormOpts;

{**********************************************************************}
{                                                                      }
{   FormOpts                                                           }
{                                                                      }
{   options dialog                                                     }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls,
  FormDlgs, Menus, ComCtrls, ExtCtrls;

type
  TFrmOpts = class(TFrmDlgs)
    PageControl: TPageControl;

    TabTabs: TTabSheet;
    GrpTabs: TGroupBox;
    ChkTabExpand: TCheckBox;
    LblTabWidth: TLabel;
    EdtTabWidth: TEdit;
    UpDnWidth: TUpDown;
    LblTabType: TLabel;
    CmbTabTypes: TComboBox;
    GrpVarTabs: TGroupBox;
    EdtVarTabs: TEdit;

    TabLineTerm: TTabSheet;
    GrpLineTerm: TRadioGroup;

    TabBlocks: TTabSheet;
    GrpIndentBy: TGroupBox;
    ChkIndentByTab: TCheckBox;
    LblIndentWidth: TLabel;
    EdtIndentBy: TEdit;
    UpDnIndentBy: TUpDown;
    GrpBlockTypes: TGroupBox;
    ChkBlockPersist: TCheckBox;
    ChkBlockOverwrite: TCheckBox;

    TabText: TTabSheet;
    GrpTextEntry: TGroupBox;
    LblIdent: TLabel;
    CmbIdent: TComboBox;
    LblTemplates: TLabel;
    CmbTemplates: TComboBox;
    GrpWordWrap: TGroupBox;
    ChkWordWrap: TCheckBox;
    LblRightMrg: TLabel;
    EdtLeftMrg: TEdit;
    UpDnLeftMrg: TUpDown;
    LblLeftMrg: TLabel;
    EdtRightMrg: TEdit;
    UpDnRightMrg: TUpDown;

    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EdtVarTabsExit(Sender: TObject);
    procedure ChkBlockPersistClick(Sender: TObject);
    procedure ChkIndentByTabClick(Sender: TObject);
    procedure ChkBlockOverwriteClick(Sender: TObject);
  private
    fOptsList: TStrings;
    fDikOpts: Boolean;
    function TestVarTabs: Boolean;
    function EncodeVarTabs: string;
    function DecodeVarTabs(S: string): string;
    procedure ReadOptions;
    procedure WriteOptions;
  public
    procedure SetOptsList(ID: Integer);
  end;

function WinDlgOpts(OptsID: Integer; DikOpts: Boolean; var PageIndex: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  Common, FormMsgBox;

type
  TByteSet = set of Byte;
  PByteSet = ^TByteSet;

resourcestring
  rsNoPermBlock = 'The UI does not use persistent blocks. ' +
                  'Input always overwrites marked blocks.';
  rsInvVarTabs  = 'Invalid variable tab stops';

{$R *.dfm}

{----------------------------------------------------------------------}
{   external interface                                                 }
{----------------------------------------------------------------------}

function WinDlgOpts(OptsID: Integer; DikOpts: Boolean; var PageIndex: Integer): Integer;
var
  Form: TFrmOpts;
begin
  Form := TFrmOpts.Create(nil);
  try
    try
      with Form do
      begin
        if not DikOpts then
        begin
          TabBlocks.TabVisible := False;
//          GrpIndentBy.Visible := False;
//          GrpBlockTypes.Visible := False;
        end;
        PageControl.ActivePageIndex := PageIndex;
        fDikOpts := DikOpts;
        SetOptsList(OptsID);
        Result := Execute;
        PageIndex := PageControl.ActivePageIndex;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}
{   TFrmOpts: public interface                                         }
{----------------------------------------------------------------------}

procedure TFrmOpts.SetOptsList(ID: Integer);
begin
  fOptsList := ValidateStrings(ID);
end;

{----------------------------------------------------------------------}
{   TFrmOpts: verifying VarTabs                                        }
{----------------------------------------------------------------------}

function TFrmOpts.TestVarTabs: Boolean;
begin
  Result := False;
  try
    EncodeVarTabs;
    Result := True;
  except
    DlgMsgBox(rsError, rsInvVarTabs);
    PageControl.ActivePageIndex := 0;
    EdtVarTabs.SetFocus;
  end;
end;

function TFrmOpts.EncodeVarTabs: string;
var
  S: string;
  I,J: Integer;
  T: PByteSet;
begin
  SetLength(Result, 32);
  for I := 1 to 32 do Result[I] := #0;
  T := PByteSet(PChar(Result));
  S := TrimLeft(EdtVarTabs.Text);
  while S <> '' do
  begin
    J := StrToInt(Tokenize(' ',S));
    if J > 255 then raise Exception.Create('');
    Include(T^, J);
    S := TrimLeft(S);
  end;
end;

function TFrmOpts.DecodeVarTabs(S: string): string;
var
  I: Integer;
  T: PByteSet;
begin
  Result := '';
  T := PByteSet(PChar(S));
  for I := 0 to 255 do
    if I in T^ then
      Result := Result + IntToStr(I) + ' ';
  Result := Copy(Result, 1, Length(Result)-1);
end;

{----------------------------------------------------------------------}
{   TFrmOpts: getting and settting the options                         }
{----------------------------------------------------------------------}

procedure TFrmOpts.ReadOptions;
begin
  ChkTabExpand.Checked := Boolean(StrToInt(fOptsList[0]));
  UpDnWidth.Position := StrToInt(fOptsList[1]);
  CmbTabTypes.ItemIndex := StrToInt(fOptsList[2]);
  EdtVarTabs.Text := DecodeVarTabs(fOptsList[3]);
  GrpLineTerm.ItemIndex := StrToInt(fOptsList[4]);
  // FileTermination := StrToInt(fOptsList[5]);
  if fDikOpts then
  begin
//    ChkBlockPersist.Checked := Boolean(StrToInt(fOptsList[6]));
//    ChkBlockOverwrite.Checked := Boolean(StrToInt(fOptsList[7]));
    ChkBlockPersist.Checked := False;
    ChkBlockOverwrite.Checked := True;
    ChkBlockPersistClick(Self);
    UpDnIndentBy.Position := StrToInt(fOptsList[8]);
    ChkIndentByTab.Checked := UpDnIndentBy.Position = 0;
  end;
  CmbIdent.ItemIndex := StrToInt(fOptsList[9]);
  CmbTemplates.ItemIndex := StrToInt(fOptsList[10]);
  ChkWordWrap.Checked := Boolean(StrToInt(fOptsList[11]));
  UpDnLeftMrg.Position := StrToInt(fOptsList[12]);
  UpDnRightMrg.Position := StrToInt(fOptsList[13]);
end;

procedure TFrmOpts.WriteOptions;
begin
  fOptsList[0] := IntToStr(Integer(ChkTabExpand.Checked));
  fOptsList[1] := IntToStr(UpDnWidth.Position);
  fOptsList[2] := IntToStr(CmbTabTypes.ItemIndex);
  fOptsList[3] := EncodeVarTabs;
  fOptsList[4] := IntToStr(GrpLineTerm.ItemIndex);
  // fOptsList[5] := IntToStr(FileTermination);
  if fDikOpts then
  begin
    fOptsList[6] := IntToStr(Integer(ChkBlockPersist.Checked));
    fOptsList[7] := IntToStr(Integer(ChkBlockOverwrite.Checked));
    fOptsList[8] := IntToStr(UpDnIndentBy.Position);
  end;
  fOptsList[9] := IntToStr(CmbIdent.ItemIndex);
  fOptsList[10] := IntToStr(CmbTemplates.ItemIndex);
  fOptsList[11] := IntToStr(Integer(ChkWordWrap.Checked));
  fOptsList[12] := IntToStr(UpDnLeftMrg.Position);
  fOptsList[13] := IntToStr(UpDnRightMrg.Position);
end;

{----------------------------------------------------------------------}
{   TFrmOpts: form response                                            }
{----------------------------------------------------------------------}

procedure TFrmOpts.FormShow(Sender: TObject);
begin
  ReadOptions;
end;

procedure TFrmOpts.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  inherited;
  if ModalResult <> idCancel then
  begin
    CanClose := TestVarTabs;
    if CanClose then WriteOptions;
  end;
end;

procedure TFrmOpts.EdtVarTabsExit(Sender: TObject);
begin
  if ActiveControl <> BtnCancel then TestVarTabs;
end;

{----------------------------------------------------------------------}
{   TFrmOpts: button response                                          }
{----------------------------------------------------------------------}

procedure TFrmOpts.ChkBlockPersistClick(Sender: TObject);
begin
  ChkBlockOverwrite.Enabled := not ChkBlockPersist.Checked;
  ChkBlockOverwrite.Checked := not ChkBlockPersist.Checked;
end;

procedure TFrmOpts.ChkBlockOverwriteClick(Sender: TObject);
begin
  if not ChkBlockOverwrite.Checked then DlgMsgBox(PChar(Caption), rsNoPermBlock);
  ChkBlockPersist.Checked := False;
  ChkBlockOverwrite.Checked := True;
end;

procedure TFrmOpts.ChkIndentByTabClick(Sender: TObject);
begin
  LblIndentWidth.Enabled := not ChkIndentByTab.Checked;
  EdtIndentBy.Enabled := not ChkIndentByTab.Checked;
  UpDnIndentBy.Enabled := not ChkIndentByTab.Checked;
  UpDnIndentBy.Position := 0;
end;

end.
