unit FormTodoList;

{**********************************************************************}
{                                                                      }
{   FormHist                                                           }
{                                                                      }
{   display and manage a list of todo items                            }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Menus, StdCtrls,
  ComCtrls, FormDlgs, Graphics;

type
  TFrmTodoList = class(TFrmDlgs)
    ListView: TListView;
    PopupMenu: TPopupMenu;
    CmdGoto: TMenuItem;
    N5: TMenuItem;
    CmdEdit: TMenuItem;
    CmdDisplay: TMenuItem;
    CmdRemove: TMenuItem;
    N6: TMenuItem;
    CmdMark: TMenuItem;
    CmdMarkDone: TMenuItem;
    CmdMarkTodo: TMenuItem;
    CmdSortBy: TMenuItem;
    CmdSortByLine: TMenuItem;
    CmdSortByPriority: TMenuItem;
    CmdSortByOwner: TMenuItem;
    CmdSortByCategory: TMenuItem;
    CmdSortByItem: TMenuItem;
    N7: TMenuItem;
    CmdSetup: TMenuItem;
    procedure ListViewDeletion(Sender: TObject; Item: TListItem);
    procedure ListViewChanging(Sender: TObject;
      Item: TListItem; Change: TItemChange; var AllowChange: Boolean);
    procedure ListViewChange(
      Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure ListViewCompare(Sender: TObject;
      Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure ListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListViewDblClick(Sender: TObject);
    procedure ListViewKeyDown(
      Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListViewCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure CmdGotoClick(Sender: TObject);
    procedure CmdEditClick(Sender: TObject);
    procedure CmdDisplayClick(Sender: TObject);
    procedure CmdRemoveClick(Sender: TObject);
    procedure CmdMarkDoneClick(Sender: TObject);
    procedure CmdMarkTodoClick(Sender: TObject);
    procedure CmdSortByItemClick(Sender: TObject);
    procedure CmdSetupClick(Sender: TObject);
  private
    SortColumn: Integer;
    ItemChecked: Boolean;
  public
    procedure SetControls(ID: Integer; SelIndex: Integer);
  end;

function WinDlgTodoList(DataID: Integer;
  var SelIndex: Integer; var X, Y, SortCol: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

uses
  Common;

{$R *.dfm}

{----------------------------------------------------------------------}

type
  PItemData = ^TItemData;
  TItemData = record
    Index: Integer;
    ItemLine: Integer;
  end;

{----------------------------------------------------------------------}

function WinDlgTodoList(DataID: Integer;
  var SelIndex: Integer; var X, Y, SortCol: Integer): Integer;
var
  Form: TFrmTodoList;
begin
  Form := TFrmTodoList.Create(nil);
  try
    try
      with Form do
      begin
        SetControls(DataID, SelIndex-1);
        ListView.CustomSort(nil, SortCol);
        Result := Execute(X,Y);
        SortCol := SortColumn;
        X := Left;
        Y := Top;
        if ListView.Selected = nil then
          SelIndex := 0
        else
          SelIndex := PItemData(ListView.Selected.Data).Index + 1;
      end;
    except
      Result := Form.LastError;
    end;
  finally
    Form.Free;
  end;
end;

{----------------------------------------------------------------------}

procedure TFrmTodoList.SetControls(ID: Integer; SelIndex: Integer);
var
  S: string;
  P: string;
  I: Integer;
  List: TStrings;
  SelItem: TListItem;
  NewItem: TListItem;
  ItemData: PItemData;
begin
  List := ValidateStrings(ID);
  SelItem := nil;
  for I := 0 to List.Count-1 do
  begin
    NewItem := ListView.Items.Add;
    if I = SelIndex then SelItem := NewItem;
    S := List[I];
    with NewItem do
    begin
      // caption (aka line number)
      P := Trim(Copy(S,1,5));
      Caption := P;
      // add item's data (index and numeric line number)
      New(ItemData);
      Data := ItemData;
      ItemData.Index := I;
      try
        ItemData.ItemLine := StrToInt(P); // aborts on invalid format
      except
        Error(slInvalidIndex);
        Exit;
      end;
      // priority or X for DONE items
      P := Copy(S,7,6);
      if P = ' DONE ' then
      begin
        Checked := True;
        SubItems.Add('');
      end else
        SubItems.Add(Trim(P));
      // remaining fields
      SubItems.Add(Trim(Copy(S,13,8)));
      SubItems.Add(Trim(Copy(S,22,16)));
      SubItems.Add(Trim(Copy(S,39,255)));
    end;
  end;
  if SelItem <> nil then
    with SelItem do
    begin
      Focused := True;
      Selected := True;
    end;
end;

{----------------------------------------------------------------------}

procedure TFrmTodoList.ListViewDeletion(Sender: TObject; Item: TListItem);
begin
  Dispose(PItemData(Item.Data));
end;

procedure TFrmTodoList.ListViewChanging(Sender: TObject;
  Item: TListItem; Change: TItemChange; var AllowChange: Boolean);
begin
  ItemChecked := Item.Checked;
end;

procedure TFrmTodoList.ListViewChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
var
  Sel: Boolean;
begin
  Sel := ListView.Selected <> nil;
  CmdGoto.Enabled := Sel;
  CmdEdit.Enabled := Sel;
  CmdDisplay.Enabled := Sel;
  if Item.Checked <> ItemChecked then
  begin
    Item.Focused := True;
    Item.Selected := True;
    if Item.Checked then
      ModalResult := idTodoDone
    else
      ModalResult := idTodoTodo;
  end;
end;

procedure TFrmTodoList.ListViewCompare(Sender: TObject;
  Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  L1, L2: Integer;
begin
  SortColumn := Data;
  if Data = 0 then
  begin
    L1 := PItemData(Item1.Data).ItemLine;
    L2 := PItemData(Item2.Data).ItemLine;
    if L1 < L2 then
      Compare := -1
    else if L1 > L2 then
      Compare := +1
    else
      Compare := 0;
  end else
  begin
    Compare := AnsiCompareText(Item1.SubItems[Data-1], Item2.SubItems[Data-1]);
  end;
end;

procedure TFrmTodoList.ListViewColumnClick(Sender: TObject; Column: TListColumn);
begin
  ListView.CustomSort(nil, Column.Index);
end;

procedure TFrmTodoList.ListViewDblClick(Sender: TObject);
begin
  ModalResult := idTodoGoto;
end;

procedure TFrmTodoList.ListViewKeyDown(
  Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then ModalResult := idCancel;
end;

procedure TFrmTodoList.ListViewCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
var
  FS: TFontStyles;
begin
  FS := Sender.Canvas.Font.Style;
  if Item.Checked then
    Include(FS, fsStrikeOut)
  else
    Exclude(FS, fsStrikeOut);
  Sender.Canvas.Font.Style := FS;
end;

{----------------------------------------------------------------------}

procedure TFrmTodoList.CmdGotoClick(Sender: TObject);
begin
  ModalResult := idTodoGoto;
end;

procedure TFrmTodoList.CmdEditClick(Sender: TObject);
begin
  ModalResult := idTodoEdit;
end;

procedure TFrmTodoList.CmdDisplayClick(Sender: TObject);
begin
  ModalResult := idTodoUpdate;
end;

procedure TFrmTodoList.CmdRemoveClick(Sender: TObject);
begin
  ModalResult := idTodoRemove;
end;

procedure TFrmTodoList.CmdMarkDoneClick(Sender: TObject);
begin
  ModalResult := idTodoDone;
end;

procedure TFrmTodoList.CmdMarkTodoClick(Sender: TObject);
begin
  ModalResult := idTodoTodo;
end;

procedure TFrmTodoList.CmdSortByItemClick(Sender: TObject);
begin
  ListView.CustomSort(nil, (Sender as TmenuItem).Tag);
end;

procedure TFrmTodoList.CmdSetupClick(Sender: TObject);
begin
  ModalResult := idTodoSetup;
end;

end.
