unit IncrSrchEdit;

{======================================================================}
{                                                                      }
{   unit IncrSrchEdit                                 Version 1.3      }
{                                                                      }
{   Property editors for IncrSrch unit                                 }
{                                                                      }
{   Copyright       (c) 1997-2001 by DiK                               }
{                                                                      }
{   History                                                            }
{   v1.31/28.07.01  adaption to Delphi6 units                          }
{   v1.30/11.12.98  no changes                                         }
{   v1.20/22.07.98  no changes                                         }
{   v1.10/16.03.98  new for version 1.1                                }
{                                                                      }
{======================================================================}

{$I Switches.Inc}

{======================================================================}
{   INTERFACE                                                          }
{======================================================================}

interface

uses
{$IFDEF DELPHI6}
  Windows, Messages, SysUtils, Classes, Forms, Dialogs, Controls,
  StdCtrls, Grids, DesignIntf, DesignEditors, IncrSrch;
{$ELSE}
  Windows, Messages, SysUtils, Classes, Forms, Dialogs, Controls,
  StdCtrls, Grids, DsgnIntf, IncrSrch;
{$ENDIF}

{======================================================================}
{   type definitions                                                   }
{======================================================================}

{----------------------------------------------------------------------}
{   TIncrSrchColumnsEditor                                             }
{----------------------------------------------------------------------}

type
  TIncrSrchColumnsEditor = class(TPropertyEditor)
  protected
    procedure CheckValue(MinVal, MaxVal: Integer); virtual;
    function GetColName(Index: Integer): String; virtual;
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

{======================================================================}
{   Registration                                                       }
{======================================================================}

procedure Register;

{======================================================================}
{   IMPLEMENTATION                                                     }
{======================================================================}

implementation

{======================================================================}
{   resource strings                                                   }
{======================================================================}

resourcestring
  rsBounds = 'Section column out of bounds';

{======================================================================}
{   internal types                                                     }
{======================================================================}

type
  TIntSet = set of 0..MaxGridColumn-1;

type // define this, so we can access the protected GetColCount function
  TColEditIncrSearc = class(TCustomGridIncrSearch)
  end;

{======================================================================}
{   Property editor dialogs                                            }
{======================================================================}

{----------------------------------------------------------------------}
{   class definition                                                   }
{----------------------------------------------------------------------}

type
  TFormIncrSrchColumns = class(TForm)
    List: TListBox;
    BtnOk: TButton;
    BtnCancel: TButton;
    BtnAll: TButton;
    BtnNone: TButton;
    procedure BtnAllClick(Sender: TObject);
    procedure BtnNoneClick(Sender: TObject);
  end;

{----------------------------------------------------------------------}
{   TFormHeaderColumns                                                 }
{----------------------------------------------------------------------}

{$R FormIncrSrchEdit.dfm}

procedure TFormIncrSrchColumns.BtnAllClick(Sender: TObject);
begin
  List.Perform(LB_SETSEL,1,-1);
end;

procedure TFormIncrSrchColumns.BtnNoneClick(Sender: TObject);
begin
  List.Perform(LB_SETSEL,0,-1);
end;

{======================================================================}
{   TIncrSrchColumnsEditor                                             }
{======================================================================}

{----------------------------------------------------------------------}
{   internal helper routines                                           }
{----------------------------------------------------------------------}

procedure TIncrSrchColumnsEditor.CheckValue(MinVal, MaxVal: Integer);
begin
  with TColEditIncrSearc(GetComponent(0)) do
  begin
    if not Assigned(Client) then
      raise EInvalidSearchClient.Create('');
    if (MinVal < 0) or (MaxVal < MinVal) or (GetColCount < MaxVal) then
      raise EInvalidGridColumn.Create(rsBounds);
  end;
end;

function TIncrSrchColumnsEditor.GetColName(Index: Integer): String;
begin
  Result := Format('Column %d',[Index]);
end;

{----------------------------------------------------------------------}
{   TIncrSrchColumnsEditor                                             }
{----------------------------------------------------------------------}

function TIncrSrchColumnsEditor.GetValue: string;
var
  Min,Max: Integer;

  procedure AddSubSet;
  begin
    if Min >= 0 then
    begin
      Result := Result + ',' + IntToStr(Min);
      if Max > Min then
        Result := Result + '-' + IntToStr(Max);
      Min := -1;
    end;
  end;

var
  I: Integer;
  Cols: TIntSet;
begin
  Result := '';
  Min := -1;
  Cols := TIntSet(Integer(GetOrdValue));
  for I := 0 to MaxGridColumn-1 do
  begin
    if I in Cols then
    begin
      if Min < 0 then Min := I;
      Max := I;
    end else
      AddSubSet;
  end;
  AddSubSet;
  Delete(Result,1,1);
  if Result = '' then Result := '0';
end;

procedure TIncrSrchColumnsEditor.SetValue(const Value: string);

  procedure Split(Delim: Char; var Lead,Rest: String);
  var
    P: Integer;
  begin
    P := Pos(Delim,Rest);
    if P = 0 then P := Length(Rest) + 1;
    Lead := Copy(Rest,1,P-1);
    Delete(Rest,1,P);
  end;

var
  Cols: TIntSet;
  I,Min,Max: Integer;
  StrMin,StrMax,Rest: String;
begin
  Cols := [];
  Rest := Value;
  try
    while Rest <> '' do
    begin
      Split(',',StrMax,Rest);
      Split('-',StrMin,StrMax);
      Min := StrToInt(StrMin);
      if StrMax = '' then Max := Min else Max := StrToInt(StrMax);
      CheckValue(Min,Max);
      for I := Min to Max do Include(Cols,I);
    end;
  except
    on E: EInvalidGridColumn do
    begin
      Cols := [];
      MessageDlg(E.Message,mtError,[mbOk],0);
    end;
    on E: EInvalidSearchClient do
    begin
      Cols := [];
    end;
  end;
  SetOrdValue(Integer(Cols));
end;

function TIncrSrchColumnsEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog,paRevertable];
end;

procedure TIncrSrchColumnsEditor.Edit;
var
  I,N: Integer;
  Cols: TIntSet;
  ISE: TFormIncrSrchColumns;
begin
  Cols := TIntSet(Integer(GetOrdValue));
  ISE := TFormIncrSrchColumns.Create(nil);
  try
    with TColEditIncrSearc(GetComponent(0)) do
    begin
      if not Assigned(Client) then
        raise EInvalidSearchClient.Create('');
      N := GetColCount - 1;
      for I:=0 to N do
      begin
        ISE.List.Items.Add(GetColName(I));
        ISE.List.Selected[I] := I in Cols;
      end;
      if ISE.ShowModal = mrOk then
      begin
        Cols := [];
        for I:=0 to N do
          if ISE.List.Selected[I] then Include(Cols,I);
      end
    end;
  finally
    ISE.Free;
    SetOrdValue(Integer(Cols));
  end;
end;

{======================================================================}
{   Registration                                                       }
{======================================================================}

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TGridColumns),
    TStringGridIncrSearch,'Columns',TIncrSrchColumnsEditor);
end;

end.
