unit StrList;

{**********************************************************************}
{                                                                      }
{   StrList                                                            }
{                                                                      }
{   string lists                                                       }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

interface

{----------------------------------------------------------------------}
{   libraries                                                          }
{----------------------------------------------------------------------}

uses
  SysUtils, Classes, Common;

{----------------------------------------------------------------------}
{   string lists (internal)                                            }
{----------------------------------------------------------------------}

function GetStringList(ID: Integer; var List: TStrings): Boolean;

{----------------------------------------------------------------------}
{   string lists (exported)                                            }
{----------------------------------------------------------------------}

function AllocStrList: Integer; stdcall;
procedure FreeStrList(ID: Integer); stdcall;
function IsStrListValid(ID: Integer): Integer; stdcall;
function GetStrListSize(ID: Integer): Integer; stdcall;
procedure ClearStrList(ID: Integer); stdcall;
function AddToStrList(ID: Integer; Str: PSalStr): Integer; stdcall;
function InsertIntoStrList(ID: Integer;
  Str: PSalStr; Index: Integer): Integer; stdcall;
function GetFromStrList(ID: Integer;
  Str: PSalStr; Len: Integer; Index: Integer): Integer; stdcall;

{**********************************************************************}
{   implementation                                                     }
{**********************************************************************}

implementation

{----------------------------------------------------------------------}
{   string lists, internal classes and data                            }
{----------------------------------------------------------------------}

type
  TStrListItem = class(TCollectionItem)
  private
    FStrList: TStringList;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  end;

var
  LastID: Integer = -1;
  LastList: TStringList = nil;
  StrLists: TCollection;

constructor TStrListItem.Create(Collection: TCollection);
begin
  inherited;
  FStrList := TStringList.Create;
end;

destructor TStrListItem.Destroy;
begin
  FStrList.Free;
  inherited;
end;

{----------------------------------------------------------------------}
{   string lists (internal)                                            }
{----------------------------------------------------------------------}

function GetStringList(ID: Integer; var List: TStrings): Boolean;
var
  Item: TStrListItem;
begin
  if ID <> LastID then
  begin
    Item := TStrListItem(StrLists.FindItemID(ID));
    if Item = nil then
    begin
      Result := False;
      Exit;
    end;
    LastId := ID;
    LastList := Item.FStrList;
  end;
  List := LastList;
  Result := True;
end;

{----------------------------------------------------------------------}
{   string lists (exported)                                            }
{----------------------------------------------------------------------}

function AllocStrList: Integer;
begin
  try
    with TStrListItem(StrLists.Add) do
    begin
      Result := ID;
      LastID := ID;
      LastList := FStrList;
    end;
  except
    Result := slInvalidID;
  end;
end;

procedure FreeStrList(ID: Integer);
begin
  if ID = LastID then
  begin
    LastID := -1;
    LastList := nil;
  end;
  StrLists.FindItemID(ID).Free;
end;

function IsStrListValid(ID: Integer): Integer;
begin
  if StrLists.FindItemID(ID) <> nil then
    Result := slOK
  else
    Result := slInvalidID;
end;

{----------------------------------------------------------------------}

function GetStrListSize(ID: Integer): Integer;
var
  List: TStrings;
begin
  if GetStringList(ID, List) then
    Result := List.Count
  else
    Result := slInvalidIndex;
end;

procedure ClearStrList(ID: Integer);
var
  List: TStrings;
begin
  if GetStringList(ID, List) then List.Clear;
end;

function AddToStrList(ID: Integer; Str: PSalStr): Integer;
var
  List: TStrings;
begin
  if GetStringList(ID, List) then
  begin
    List.Add(GetSalStr(Str));
    Result := slOK;
  end else
    Result := slInvalidID;
end;

function InsertIntoStrList(ID: Integer;
  Str: PSalStr; Index: Integer): Integer;
var
  List: TStrings;
begin
  if GetStringList(ID, List) then
  begin
    if (0 <= Index) and (Index <= List.Count) then
    begin
      List.Insert(Index, GetSalStr(Str));
      Result := slOK;
    end else
      Result := slInvalidIndex;
  end else
    Result := slInvalidID;
end;

function GetFromStrList(ID: Integer;
  Str: PSalStr; Len: Integer; Index: Integer): Integer;
var
  List: TStrings;
begin
  if GetStringList(ID, List) then
  begin
    if (0 <= Index) and (Index < List.Count) then
    begin
      SetSalStr(Str, Len, List[Index]);
      if Len >= Length(List[Index]) then
        Result := slOK
      else
        Result := slInvalidLength;
    end else
      Result := slInvalidIndex;
  end else
    Result := slInvalidID;
end;

{**********************************************************************}
{   initialization                                                     }
{**********************************************************************}

initialization
  StrLists := TCollection.Create(TStrListItem);
finalization
  StrLists.Free;
end.

