library WinDlgs;

{**********************************************************************}
{                                                                      }
{   WinDlgs                                                            }
{                                                                      }
{   main module, defines exported functions                            }
{                                                                      }
{   Version     v0.96/12.05.05                                         }
{   Copyright   (c) 2003-2005 by DiK                                   }
{                                                                      }
{**********************************************************************}

{----------------------------------------------------------------------}
{   Version History                                                    }
{                                                                      }
{   v0.96/12.05.05  feature complete (?)                               }
{   + preliminary doc file                                             }
{   + still more in code doc                                           }
{   + fixed visual design of dialogs                                   }
{   + fixed enhanced context menu                                      }
{   + fixed various odds and ends                                      }
{   - options dialog                                                   }
{   - bookmark list                                                    }
{   - todo manager                                                     }
{   - history list manager                                             }
{   - file comparison (tsecomp)                                        }
{                                                                      }
{   v0.92/29.01.04  third beta                                         }
{   + even more in code doc                                            }
{   + strlist caching actually working                                 }
{   - enhanced fill block (popup menu)                                 }
{   - enhanced find&do (uses external find&do)                         }
{                                                                      }
{   v0.91/19.08.03  second beta                                        }
{   + some bug fixes (preserve blocks)                                 }
{   + exhanced context menu for some edit controls                     }
{                                                                      }
{   v0.90/08.05.03  first beta                                         }
{   + more help, fixed bugs                                            }
{   + better in code doc                                               }
{   - open, save as, pick folders                                      }
{   - fill block                                                       }
{   - grep, find&do, goto line&col                                     }
{   - mcc, os commands                                                 }
{                                                                      }
{   v0.70/21.03.03  alpha version                                      }
{   + first draft                                                      }
{   + basic programming model                                          }
{   - buffer list, recent files list                                   }
{   - find, replace                                                    }
{                                                                      }
{----------------------------------------------------------------------}

{----------------------------------------------------------------------}
{   libraries                                                          }
{----------------------------------------------------------------------}

uses
  SysUtils,
  Forms,
  Common in 'Common.pas',
  StrList in 'StrList.pas',
  DlgOpen in 'DlgOpen.pas',
  DlgOpenEx in 'DlgOpenEx.pas',
  FormDlgs in 'FormDlgs.pas' {FrmDlgs},
  FormChar in 'FormChar.pas' {FrmChar},
  FormMsgBox in 'FormMsgBox.pas' {FrmMsgBox},
  FormYesNoBox in 'FormYesNoBox.pas' {FrmYesNoBox},
  FormInpBox in 'FormInpBox.pas' {FrmInpBox},
  FormFolder in 'FormFolder.pas' {FrmFolder},
  FormStrList in 'FormStrList.pas' {FrmStrList},
  FormBuffers in 'FormBuffers.pas' {FrmBuffers},
  FormRecent in 'FormRecent.pas' {FrmRecent},
  FormFind in 'FormFind.pas' {FrmFind},
  FormFindAndDo in 'FormFindAndDo.pas' {FrmFindAndDo},
  FormReplace in 'FormReplace.pas' {FrmReplace},
  FormGrep in 'FormGrep.pas' {FrmGrep},
  FormGoto in 'FormGoto.pas' {FrmGoto},
  FormBookmarks in 'FormBookmarks.pas' {FrmBookmarks},
  FormMCC in 'FormMCC.pas' {FrmMCC},
  FormOpts in 'FormOpts.pas' {FrmOpts},
  FormComp in 'FormComp.pas' {FrmComp},
  FormHist in 'FormHist.pas' {FrmHist},
  FormHistStr in 'FormHistStr.pas' {FrmHistStr},
  FormHistUser in 'FormHistUser.pas' {FrmHistUser},
  FormTodo in 'FormTodo.pas' {FrmAddTodo},
  FormTodoList in 'FormTodoList.pas' {FrmTodoList},
  FormTodoSetup in 'FormTodoSetup.pas' {FrmTodoSetup};

{----------------------------------------------------------------------}
{   initialization                                                     }
{                                                                      }
{   WinDlgInit must be called before displaying the first dialog       }
{   + WinDlgInit only initializes the VCL (window handle)              }
{   + string list handling is set up in the StrList unit               }
{----------------------------------------------------------------------}

procedure WinDlgInit(TSE: THandle); stdcall;
begin
  Application.Handle := TSE;
end;

function WinDlgVersion: Integer; stdcall;
begin
  LongRec(Result).Hi := cnVersionHi;
  LongRec(Result).Lo := cnVersionLo;
end;

{----------------------------------------------------------------------}
{   export functions                                                   }
{----------------------------------------------------------------------}

exports
  AllocStrList      index   1   name 'AllocStrList',
  FreeStrList       index   2   name 'FreeStrList',
  IsStrListValid    index   3   name 'IsStrListValid',
  GetStrListSize    index   4   name 'GetStrListSize',
  ClearStrList      index   5   name 'ClearStrList',
  AddToStrList      index   6   name 'AddToStrList',
  InsertIntoStrList index   7   name 'InsertIntoStrList',
  GetFromStrList    index   8   name 'GetFromStrList',

  WinDlgInit        index  11   name 'WinDlgInit',
  WinDlgVersion     index  12   name 'WinDlgVersion',

  WinDlgMsgBox      index  16   name 'WinDlgMsgBox',
  WinDlgYesNoBox    index  17   name 'WinDlgYesNoBox',
  WinDlgInpBox      index  18   name 'WinDlgInpBox',
  WinDlgInpBoxEx    index  19   name 'WinDlgInpBoxEx',
  WinDlgOpen        index  21   name 'WinDlgOpen',
  WinDlgOpenEx      index  22   name 'WinDlgOpenEx',
  WinDlgSaveAs      index  23   name 'WinDlgSaveAs',
  WinDlgPickFolder  index  24   name 'WinDlgPickFolder',
  WinDlgBuffers     index  25   name 'WinDlgBuffers',
  WinDlgRecent      index  26   name 'WinDlgRecent',
  WinDlgFind        index  31   name 'WinDlgFind',
  WinDlgReplace     index  32   name 'WinDlgReplace',
  WinDlgFindAndDo   index  33   name 'WinDlgFindAndDo',
  WinDlgGrep        index  34   name 'WinDlgGrep',
  WinDlgGoto        index  35   name 'WinDlgGoto',
  WinDlgBookmarks   index  36   name 'WinDlgBookmarks',
  WinDlgMCC         index  41   name 'WinDlgMCC',
  WinDlgOpts        index  51   name 'WinDlgOpts',
  WinDlgComp        index  56   name 'WinDlgComp',
  WinDlgHistories   index  61   name 'WinDlgHistories',
  WinDlgHistStrings index  62   name 'WinDlgHistStrings',
  WinDlgHistUser    index  63   name 'WinDlgHistUser',
  WinDlgAddTodo     index  66   name 'WinDlgAddTodo',
  WinDlgTodoList    index  67   name 'WinDlgTodoList',
  WinDlgTodoSetup   index  68   name 'WinDlgTodoSetup',
  WinDlgStrList     index  71   name 'WinDlgStrList';

{**********************************************************************}
{   initialization                                                     }
{**********************************************************************}

//  none

end.
